#include "ladders.h"
#include "Platform.h"


////////////////////////////////////////////////////////////////////////
//Colors  (taken from screenshot of ldmicro)
#define CommentColor 0x5978D1
#define NameColor 0x5FE055

#define White  0xFFFFFF
#define Gray   0x808080
#define Yellow 0xFFFF00
#define Black  0x000000
#define Red    0xFF0000
#define Magenta 0xFF00FF

#define NameOnColor Red
#define NameOffColor Gray
#define CursorColor Magenta


//////////////////////////////////////////////////////////////////////

//Drawing Code
#define rows 48
#define cols 128

char s[256 + 1];  //shared text buffer

char blankladder[cols +1];
char menubar[cols + 1];

#define maxverticalbars (256)

typedef struct VerticalBar
{//VerticalBar object
    char top;
    char bot;
    char col;
} VerticalBar;

VerticalBar VerticalBars[maxverticalbars];

int nVerticalBars;

int nStep; //step # within rung
#define nSteps 6 //#possible steps
#define nStepWidth  16

#define RungWidth  (nStepWidth*nSteps) //spaces between vertical bars
int nRow;
int nNextRungRow;  //trying to accout for parallel instructions
#define FirstStepCol  7
#define CoilCol  (FirstStepCol+ RungWidth-nStepWidth)//90;//column where coils start
#define LeftRailCol 5
#define RightRailCol (LeftRailCol+ RungWidth+1)

void DrawComment(CommentObject* pCO)
{
    SetForeColor24(CommentColor);
    SetBackColor24(0);
    SetRowCol(nRow, 8);
    vga.str("; ");
    vga.str(pCO->line1);
    if (strlen(pCO->line2) > 0)
    {
        SetRowCol(nRow + 1, 8);
        vga.str("; ");
        vga.str(pCO->line2);
    }
}



void DrawContact(ContactObject* pCO)
{
    if (PlcMode == EditMode)
        SetForeColor24(NameColor);
    else if ((PlcMode == SimulationMode) || (PlcMode == RunMode))
    {
        if (pCO->bValue)
            SetForeColor24(NameOnColor);
        else
            SetForeColor24(NameOffColor);
    }
    
    SetBackColor24(0);
    SetRowCol(nRow, FirstStepCol + nStepWidth * nStep+ nStepWidth/2 - (strlen(pCO->name)/2));
    sprintf(s, "%c%s", pCO->source, pCO->name);
    vga.str(s);

    //printf("Drawing Contact %s at step %d, with value=%d, series on =%d\n", pCO->name, nStep, pCO->bValue, pCO->bSeriesOn);
    //for (;;) {}


    if ((PlcMode == SimulationMode) || (PlcMode == RunMode))
    {
        if ((pCO->bSeriesOn)|| ((pCO->bOn)&&(nStep==0)))
            SetForeColor24(White);
        else
            SetForeColor24(Gray);
    }
    else if (PlcMode == EditMode)
        SetForeColor24(White);

    
    SetRowCol(nRow+1, FirstStepCol + nStepWidth * nStep);
    pCO->row = nRow + 1;
    pCO->col = FirstStepCol + nStepWidth * nStep;
    if ((PlcMode==SimulationMode)&&(pCO->bInitialValue==true)) //Draw leading carrot if InitialValue is set to 1 and in simulation mode
        if (pCO->bNegated == false)
            vga.str("------^] [------");
        else
            vga.str("------^]/[------");
    else
        if (pCO->bNegated == false)
            vga.str("-------] [------");
        else
            vga.str("-------]/[------");

}

void DrawTon(TonObject* pTon)
{
    if (PlcMode == EditMode)
        SetForeColor24(NameColor);
    else if ((PlcMode == SimulationMode) || (PlcMode == RunMode))
    {
        if (pTon->bOn)
            SetForeColor24(NameOnColor);
        else
            SetForeColor24(NameOffColor);
    }

    SetBackColor24(0);
    SetRowCol(nRow, FirstStepCol + nStepWidth * nStep + nStepWidth / 2 - (strlen(pTon->name) / 2));
    sprintf(s, "%c%s", pTon->source, pTon->name);
    vga.str(s);

    //printf("Drawing Contact %s at step %d, with value=%d, series on =%d\n", pCO->name, nStep, pCO->bValue, pCO->bSeriesOn);
    //for (;;) {}


    if ((PlcMode == SimulationMode) || (PlcMode == RunMode))
    {
        if (pTon->bSeriesOn)
            SetForeColor24(White);
        else
            SetForeColor24(Gray);
    }
    else if (PlcMode == EditMode)
        SetForeColor24(White);


    SetRowCol(nRow + 1, FirstStepCol + nStepWidth * nStep);
    pTon->row = nRow + 1;
    pTon->col = FirstStepCol + nStepWidth * nStep;
    char mess[100];
    int onms = pTon->onus / 1000;
    float ons = onms / 1000.;
    if (onms >= 1000)
        sprintf(mess, "--[TON %3.2f s]--", ons);
    else
        sprintf(mess, "--[TON %3d ms]-",onms);
    vga.str(mess);

}

void DrawToff(ToffObject* pToff)
{
    if (PlcMode == EditMode)
        SetForeColor24(NameColor);
    else if ((PlcMode == SimulationMode) || (PlcMode == RunMode))
    {
        if (pToff->bOn)
            SetForeColor24(NameOnColor);
        else
            SetForeColor24(NameOffColor);
    }

    SetBackColor24(0);
    SetRowCol(nRow, FirstStepCol + nStepWidth * nStep + nStepWidth / 2 - (strlen(pToff->name) / 2));
    sprintf(s, "%c%s", pToff->source, pToff->name);
    vga.str(s);

    //printf("Drawing Contact %s at step %d, with value=%d, series on =%d\n", pCO->name, nStep, pCO->bValue, pCO->bSeriesOn);
    //for (;;) {}


    if ((PlcMode == SimulationMode) || (PlcMode == RunMode))
    {
        if (pToff->bSeriesOn)
            SetForeColor24(White);
        else
            SetForeColor24(Gray);
    }
    else if (PlcMode == EditMode)
        SetForeColor24(White);


    SetRowCol(nRow + 1, FirstStepCol + nStepWidth * nStep);
    pToff->row = nRow + 1;
    pToff->col = FirstStepCol + nStepWidth * nStep;
    char mess[100];
    int offms = pToff->offus / 1000;
    float offs = offms / 1000.;
    if (offms >= 1000)
        sprintf(mess, "--[TOF %3.2f s]--", offs);
    else
        sprintf(mess, "--[TOF %3d ms]-", offms);
    vga.str(mess);

}

void DrawCoil(CoilObject* pCO)
{//Must be on right side (?)
    //Draw wire from last step
    if (pCO->bOn)
        SetForeColor24(White);
    else
        SetForeColor24(Gray);
    SetRowCol(nRow + 1, FirstStepCol + nStepWidth * nStep);
    while (nStep < nSteps)
    {
        //printf("NStep = % d\n", nStep);
        vga.str("----------------");
        nStep++;
    }

    //Then, draw coil
    if (PlcMode == EditMode)
        SetForeColor24(NameColor);
    else if ((PlcMode == SimulationMode) || (PlcMode == RunMode))
    {
        if (pCO->bOn)
            SetForeColor24(NameOnColor);
        else
            SetForeColor24(NameOffColor);
    }
    SetBackColor24(0);
    SetRowCol(nRow, CoilCol + nStepWidth / 2 - (strlen(pCO->name) / 2));
    sprintf(s, "%c%s", pCO->dest, pCO->name);
    vga.str(s);

    if ((PlcMode == SimulationMode)|| (PlcMode == RunMode))
    {
        if (pCO->bOn)
            SetForeColor24(White);
        else
            SetForeColor24(Gray);
    }
    else if (PlcMode == EditMode)
        SetForeColor24(White);

    SetRowCol(nRow + 1, CoilCol);
    pCO->row = nRow + 1;
    pCO->col = CoilCol;
    if (pCO->bSetOnly)
        vga.str("-------(S)------");
    else if (pCO->bResetOnly)
        vga.str("-------(R)------");
    else 
    {
        if (pCO->bNegated)
            vga.str("-------(/)------");
        else
            vga.str("-------( )------");
    }
}

void DrawPlaceholder()
{//Draw a little stub at end of blank rung
    SetForeColor24(White);
    SetRowCol(nRow + 1, FirstStepCol + nStepWidth * nStep);
    vga.str("--");
}

void WireToEnd(int nStartStep)
{
    //At end of series, draw wire to end
    SetRowCol(nRow + 1, FirstStepCol + nStepWidth * (nStartStep));
    int n = nStartStep;
    while (n < (nSteps - 1))
    {
        //printf("NStep = % d\n", nStep);
        vga.str("----------------");
        n++;
    }
}

void EraseWireToEnd(int startrow, int nStartStep)
{
    //At end of series, draw wire to end
    for (int r = startrow + 1; r < 120; r += 3)
    {
        SetRowCol(r, FirstStepCol + nStepWidth * (nStartStep));
        int n = nStartStep;
        while (n < nSteps)
        {
            //printf("NStep = % d\n", nStep);
            vga.str("                ");
            n++;
        }
    }

}

int nLowestRow;

int DrawParallel(int nRung, ProgramObject* p, int index)
{//Draw parallel objects
    int nStartRow = nRow;
    int nStartStep = nStep;
    int nEndStep = nStep + 1;
    index++;

    EraseWireToEnd(nRow, nStep);

   // printf("Drawing Parallel at start step %d and end step %d\n", nStartStep, nEndStep);

    int WireStartStep = nEndStep;

    InstructionObject* pIO;
    do
    {
        //nRow = nNextParallelRow;

        pIO = p->p[index];
        //WireStartStep = nEndStep;


        switch (pIO->type)
        {
        case CommentType:
            DrawComment((CommentObject*)pIO);
            break;
        case ContactType:
            DrawContact((ContactObject*)pIO);
            WireToEnd(nStep + 1);
            break;
        case TonType:
            DrawTon((TonObject*)pIO);
            WireStartStep = nStep + 1; //reset
            WireToEnd(nStep + 1);
            break;
        case ToffType:
            DrawToff((ToffObject*)pIO);
            WireStartStep = nStep + 1; //reset
            WireToEnd(nStep + 1);
            break;
        case CoilType:
            DrawCoil((CoilObject*)pIO);
            WireToEnd(nStep + 1);
            break;
        case EndType:
            //if (nStep == 0)
            //    DrawEnd();
            
            nRow -= 3;
            break;
        case ParallelType:
            //printf("Drawing Parallel inside Parallel at step %d\n", nStep);

            index = DrawParallel(nRung, p, index+1) - 1;
            //nStartRow = nLastRow+3;
            //nStartRow = nRow+3;
            //nNextParallelRow = nRow;
            //nRow += 3;
            if (nRow > nLowestRow)
                nLowestRow = nRow;

            break;
        case SeriesType:
           // nStep = nStartStep;
            //printf("Drawing series at step %d\n", nStep);
            index = DrawSeries(nRung, p, index+1) -1;
            nStep--;
            
            if (nStep > nEndStep)
            {
                nEndStep = nStep;
                //if this is longer than previous, need to fill in with wire somehow?
            }
            WireToEnd(nStep + 1);
            nStep = nStartStep;

            

            //printf("Wire start step=%d\n", WireStartStep);
            //for (;;) {}
            break;
        default:
            break;
        }
        index++;


        ////Maybe draw wire to the end and then delete unnecessary later?
        ////SetForeColor24(White);
        //SetRowCol(nRow + 1, FirstStepCol + nStepWidth * (WireStartStep));
        //int n = WireStartStep;
        //while (n < nSteps)
        //{
        //    //printf("NStep = % d\n", nStep);
        //    vga.str("----------------");
        //    n++;
        //}
        


        //nStep++;
        
        nRow+=3;
        //nNextParallelRow += 3;

        //printf(" now nStep=%d\n", nStep);
    } while (pIO->type != EndType);



    nRow -= 3;
    //nNextParallelRow -= 3;



    //Define right side vertical bar
    VerticalBars[nVerticalBars].col = FirstStepCol + nStepWidth * nEndStep;
    VerticalBars[nVerticalBars].top = nStartRow + 1;
    VerticalBars[nVerticalBars].bot = nRow + 1;
    nVerticalBars++;

    //Define left side vertical bar
    if (nStep > 0)
    {
        VerticalBars[nVerticalBars].col = FirstStepCol + nStepWidth * (nStep);
        VerticalBars[nVerticalBars].top = nStartRow + 1;
        VerticalBars[nVerticalBars].bot = nRow + 1;
        nVerticalBars++;
    }

    ////Now draw vertical line at end of parallels and "+" binders
    //SetForeColor24(White);
    //for (int y = nStartRow; y <= nRow ; y++)
    //{        
    //    SetRowCol(y + 1, FirstStepCol + nStepWidth * (nStep+1));
    //    if ((y==nStartRow)||(y== nRow))
    //        vga.str("+");
    //    else
    //        vga.str("|");
    //}

    nStep = nEndStep;
    
    ////Now erase excess wires
    //
    //for (int y = nStartRow; y <= nRow; y += 3)
    //{
    //    SetForeColor24(White);
    //    SetRowCol(y+ 1, FirstStepCol + nStepWidth * nStep);
    //    int n = nStep;
    //    while (n < nSteps)
    //    {
    //        //printf("erasing NStep = %d at y=%d\n", n,y);
    //        vga.str("                ");
    //        n++;
    //    }
    //}




    
    //printf("Parallel done with step= %d\n", nStep);

    if ((nRow + 3) > nNextRungRow)
        nNextRungRow = nRow + 3;
   

    //printf("Parallel Setting nNextRungRow=%d\n", nNextRungRow);
    //nRow = nStartRow;
    return index;
}

int DrawSeries(int nRung, ProgramObject* p, int index)
{//Draw series of objects

    int tmp = nRow;
    int nLowestRow = nRow;

    //int nRungsDeep = 1;
    InstructionObject* pIO;
    do
    {
        pIO = p->p[index];

        switch (pIO->type)
        {
        case CommentType:
            DrawComment((CommentObject*)pIO);
            break;
        case ContactType:
            DrawContact((ContactObject*)pIO);
            break;
        case TonType:
            DrawTon((TonObject*)pIO);
            break;
        case ToffType:
            DrawToff((ToffObject*)pIO);
            break;
        case CoilType:
            DrawCoil((CoilObject*)pIO);
            break;
        case EndType:
            //if (nStep==0)
            //    DrawEnd();
            //printf("Drawing End.\n");
            break;
        case PlaceholderType:
            //if (nStep==0)
            DrawPlaceholder();
            //printf("Drawing End.\n");
            break;
        case ParallelType:
            //printf("drawing parallel at index=%d\n", index);
            tmp = nRow;
            index = DrawParallel(nRung, p, index) - 1;
            EraseWireToEnd(tmp + 3, nStep);
            if (nRow > nLowestRow)
                nLowestRow = nRow;
            nRow = tmp;
            nStep--;
            break;
        case SeriesType:
            //printf("Drawing Series inside series...\n");
            //nStep--;
            break;
        break;        default:
           // printf("Unrecognized type in series...\n");
            break;
        }
        index++;
        nStep++;
        //printf(" now nStep=%d\n", nStep);
    } while (pIO->type != EndType);


    nRow = nLowestRow;// +3;

    //printf("done drawing a series, index=%d\n", index);
    return index;
}

int DrawRung(int nRung, ProgramObject* p, int index)
{//Draw a rung of the ladder

    //Draw top line
    SetForeColor(255, 255, 255);
    SetBackColor(0, 0, 0);
    char mess[100];
    SetRowCol(nRow + 1, 2);
    sprintf(mess, "%04d", nRung );
    vga.str(mess);

    nNextRungRow = nRow + 3;

    nStep = 0;
    index = DrawSeries(nRung, p, index);
    return index;
}

void DrawEndRung(int nRung)
{//Draw bottom rung of the ladder

    //Draw top line
    SetForeColor24(White);
    SetBackColor24(Black);
    char mess[100];    
    sprintf(mess, "%d", nRung);
    SetRowCol(nRow + 1, 2 + 4 - strlen(mess));
    vga.str(mess);

    SetForeColor24(White);
    SetRowCol(nRow + 1, FirstStepCol);
    for (int i=0;i<nSteps;i++)
    {
        if (i==0)
            vga.str("-----[END]------");
        else
            vga.str("----------------");
    }
}

void DrawVerticalBars()
{
    //printf("drawing vertical bars...\n");
    //Now draw vertical line at end of parallels and "+" binders
    SetForeColor24(White);
    for (int n = 0; n < nVerticalBars; n++)
    {
        char col = VerticalBars[n].col;
        char top = VerticalBars[n].top;
        char bot = VerticalBars[n].bot;
        for (char y = top; y <= bot; y++)
        {
            SetRowCol(y, col);
            if ((y == top) || (y == bot))
                vga.str("+");
            else
                vga.str("|");
        }
    }

}


void DrawProgram(ProgramObject* p)
{
    int nRung = 0;
    int index = 0;
    nNextRungRow = nRow;
    nVerticalBars = 0;

    while (index<MaxInstructions)
    {
        nRow = nNextRungRow;

        InstructionObject* in = p->p[index++];
        if (in == 0)
        {//End of program?
            //printf("End of program..\n");
            DrawEndRung(nRung);
            DrawVerticalBars(); //when all done...
            return;
        }
        if (in->type != 0)
            //printf("Next instruction type = %d, index=%d\n", in->type, index);

            if (in->type == RungType)
            {
                //printf("Drawing Rung# %d\n", nRung + 1);

                //printf("Drawing next rung at row=%d\n", nRow);
                index = DrawRung(++nRung, p, index);
                nRow += 3;
            }


    }

}




void SetForeColor(uint8_t r, uint8_t g, uint8_t b)
{
    sprintf(s, "\x1B[38;2;%d;%d;%dm", r, g, b);
    vga.str(s);
}

void SetForeColor24(int c)
{
    uint8_t r;
    uint8_t g;
    uint8_t b;
    r = c >> 16 & 0xff;
    g = c >> 8 & 0xff;
    b = c & 0xff;
    SetForeColor(r, g, b);
}

void SetBackColor(uint8_t r, uint8_t g, uint8_t b)
{
    sprintf(s, "\x1B[48;2;%d;%d;%dm", r, g, b);
    vga.str(s);
}

void SetBackColor24(int c)
{
    uint8_t r;
    uint8_t g;
    uint8_t b;
    r = c >> 16 & 0xff;
    g = c >> 8 & 0xff;
    b = c & 0xff;
    SetBackColor(r, g, b);
}

void SetRowCol(int row, int col)
{
    sprintf(s, "\x1B[%d;%dH", row, col);
    vga.str(s);
}

void MakeBlankLadder()
{
    //make blank ladder
    for (int i = 0; i < cols; i++)
    {
        if ((i == LeftRailCol) || (i == RightRailCol))
            blankladder[i] = '|';
        else
            blankladder[i] = ' ';
    }
}

void MakeMenuBar()
{
    //make blank ladder
    for (int i = 0; i < cols; i++)
    {
        menubar[i] = ' ';
    }
    char* menu = "P2-PLC:      RUN        STOP       PAUSE       STEP      SIMULATE       EDIT       LOAD        SAVE";
    memcpy(menubar, menu, strlen(menu));
}


void HandleMouseUp(ProgramObject* p, int x, int y)
{
    int index = 0;
    int nInstructionWidth = 15;

    while (index < MaxInstructions)
    {//See if mouse clicked on contact
        InstructionObject* in = p->p[index++];
        if (in == 0)
        {//End of program?
            //printf("End of program..\n");

            break;
        }
        if (in->type == ContactType)
        {
            ContactObject* pCO = (ContactObject*)in;
            if ((pCO->source == 'X') || (pCO->source == 'Y'))
            {//See if clicked on this contact 
                //printf("Checking %s for mouse at %d,%d; row=%d,col=%d\n", pCO->name, x, y, pCO->row, pCO->col);
                if (pCO->row == y)
                {
                    int c = pCO->col;
                    if ((x > c) && (x <= (c + 15)))
                    {//This contact was clicked on
                        
                        IO* io = (IO*) pCO->pI;

                        if (io->value == true)
                            io->value = false;
                        else
                            io->value = true;

                        //printf("Clicked on %s; mouse at %d,%d; row=%d,col=%d, value=%d\n", pCO->name, x, y, pCO->row, pCO->col, io->value);
                        return;// break; //stop looking

                    }
                    
                }
            }
            else if (pCO->source == 'R')
            {//See if clicked on this internal relay contact
                //printf("Checking %s for mouse at %d,%d; row=%d,col=%d\n", pCO->name, x, y, pCO->row, pCO->col);
                if (pCO->row == y)
                {
                    int c = pCO->col;
                    if ((x > c) && (x <= (c + 15)))
                    {//This contact was clicked on

                        Internal* intern = (Internal*)pCO->pI;

                        if (intern->value == true)
                            intern->value = false;
                        else
                            intern->value = true;

                        //printf("Clicked on %s; mouse at %d,%d; row=%d,col=%d, value=%d\n", pCO->name, x, y, pCO->row, pCO->col, intern->value);
                        return;//break; //stop looking

                    }

                }
            }
        }

        else if (in->type == CoilType)
        {
            CoilObject* pCO = (CoilObject*)in;
            if ((pCO->dest == 'X') || (pCO->dest == 'Y'))
            {//See if clicked on this coil
                //printf("Checking %s for mouse at %d,%d; row=%d,col=%d\n", pCO->name, x, y, pCO->row, pCO->col);
                if (pCO->row == y)
                {
                    int c = pCO->col;
                    if ((x > c) && (x <= (c + 15)))
                    {//This contact was clicked on

                        IO* io = (IO*)pCO->pI;

                        if (io->value == true)
                            io->value = false;
                        else
                            io->value = true;

                        //printf("Clicked on %s; mouse at %d,%d; row=%d,col=%d, value=%d\n", pCO->name, x, y, pCO->row, pCO->col, io->value);
                        return;//break; //stop looking

                    }

                }
            }
            else if (pCO->dest == 'R')
            {//See if clicked on this internal relay coil
                //printf("Checking %s for mouse at %d,%d; row=%d,col=%d\n", pCO->name, x, y, pCO->row, pCO->col);
                if (pCO->row == y)
                {
                    int c = pCO->col;
                    if ((x > c) && (x <= (c + 15)))
                    {//This contact was clicked on

                        Internal* intern = (Internal*)pCO->pI;

                        if (intern->value == true)
                            intern->value = false;
                        else
                            intern->value = true;

                        if (pCO->bOn == true)
                            pCO->bOn = false;
                        else
                            pCO->bOn = true;

                        //printf("Clicked on %s; mouse at %d,%d; row=%d,col=%d, value=%d\n", pCO->name, x, y, pCO->row, pCO->col, intern->value);
                        return;//break; //stop looking

                    }

                }
            }
        }


    }

    //See if I/O list was clicked on and toggle if was
    int nio = 0;
    while (nio < MaxIO)
    {//Scan io for matching row/col
        IO* io = IoFromFile.p[nio++];
        if (io == 0)
            break;
        if (y==io->row)
        {
            int c = io->col;
            if ((x >= c) && (x <= (c + 15)))
            {//This io was clicked on
                if (io->value == true)
                    io->value = false;
                else
                    io->value = true;
                return;
            }
        }

    }

    //See if I/O list was clicked on and toggle if was
    int nintern = 0;
    while (nintern < MaxInternal)
    {//Scan intern for matching row/col
        Internal* intern = InternalFromFile.p[nintern++];
        if (intern == 0)
            break;
        if (y == intern->row)
        {
            int c = intern->col;
            if ((x >= c) && (x <= (c + 15)))
            {//This io was clicked on
                if (intern->value == true)
                    intern->value = false;
                else
                    intern->value = true;
                return;
            }
        }

    }
    //_waitms(5000);
}


typedef struct Ansi4 {
    char  fgcolor;
    char  bgcolor;
    char  curchar;
    char  cureff;
} Ansi4;

void TransmitScreen()
{
    for (;;)
    {//send screen forever
        Ansi4* pChar = vga.GetBuffer();
        Ansi4* pLastChar = pChar;

        printf("\x1B[?25l"); //hide cursor
        //printf("\x1B[2J"); //clear screen
        _waitus(100);

        for (int y = 0; y < rows; y++)
        {
            
            
            char fgcolor = pChar->fgcolor;
            char bgcolor = pChar->bgcolor;
            printf("\x1B[38;5;%dm", fgcolor); //Set foreground color
            printf("\x1B[48;5;%dm", bgcolor); //Set background color
            printf("\x1B[%d;%dH", y + 1, 1); //Set to this row, first column
            _waitus(100);
            for (int x = 0; x < cols; x++)
            {
                char newchar = pChar->curchar;
                char newfg = pChar->fgcolor;
                char newbg = pChar->bgcolor;
                if (newbg != bgcolor)
                {//New background color?
                    bgcolor = newbg;
                    printf("\x1B[48;5;%dm", bgcolor); //Set background color
                    _waitus(100);
                }
                if ((newfg != fgcolor) && (newchar != ' '))
                {//New foreground color and not a space?
                    fgcolor = newfg;
                    printf("\x1B[38;5;%dm", fgcolor); //Set background color
                    _waitus(100);
                }
                putchar(newchar);
                //printf("%c", newchar);
                //buf[x] = pChar->curchar;
                pChar += 1;
                _waitus(100);
            }
            //printf("\n");
            //_waitms(10);
            _waitus(100);
        }
    }

}

void PrepareBlankLadder()
{
    MakeBlankLadder();
    MakeMenuBar();

    vga.str("\x1B[25l"); //hide cursor
    vga.hideCursor();

    //vga.str("\x1B[2J"); //clear screen
    nRow = 1;
    //Draw menu bar
    //vga.str("\x1B[1;1H"); //home
    SetRowCol(nRow++, 1);

    vga.str("\x1B[38;2;255;255;0m"); //foreground
    vga.str("\x1B[48;2;0;0;128m"); // background
    vga.str(menubar);


    //Draw blank ladder
    SetForeColor(255, 255, 255);
    SetBackColor(0, 0, 0);
    char mess[100];
    SetRowCol(2, 1);
    for (int i = 2; i < (rows - 5); i++)
    {
        //SetRowCol(i, 1);
        //sprintf(mess, "row %d", i);
        vga.str(blankladder);
    }

    vga.CopyScreenToBlankLadder();
}

void UpdateScreen(ProgramObject* pProgram)
{

    vga.str("\x1B[25l"); //hide cursor
    vga.hideCursor();

    vga.CopyBlankLadderToScreen();  //update screen to starting condition more quickly at the expense of memory for another screen buffer...

    
    //Then, draw the ladder
    nRow = 3; 
    DrawProgram(pProgram);

    int nStatusCol = 106;
    char mess[100];

    //Show loop;
    nRow = 4;
    SetRowCol(nRow++, nStatusCol);
    vga.str("\x1B[38;2;255;255;255m"); //foreground
    vga.str("\x1B[48;2;0;0;128m"); // background
    sprintf(mess, "Loop# = %d", nLoop);
    vga.str(mess);
    nRow++;

    //Then, show I/O status
    SetRowCol(nRow++, nStatusCol);
    vga.str("\x1B[38;2;255;255;255m"); //foreground
    vga.str("\x1B[48;2;0;0;128m"); // background
    vga.str("I/O STATUS:");
    SetRowCol(nRow++, nStatusCol);
    vga.str("NAME          PIN VAL");
    vga.str("\x1B[38;2;255;255;255m"); //foreground
    vga.str("\x1B[48;2;0;0;0m"); // background

    
    int nio = 0;
    while (nio < MaxIO)
    {//Update all IO states
        IO* io = IoFromFile.p[nio++];
        if (io == 0)
            break;
        SetRowCol(nRow, nStatusCol);
        io->row = nRow; //Save row and col so can toggle in simulation mode
        io->col = nStatusCol;
        sprintf(mess, "%c %s", io->type, io->name);
        vga.str(mess);
        SetRowCol(nRow, nStatusCol+14);
        sprintf(mess, "%d", io->pin);
        vga.str(mess);
        SetRowCol(nRow, nStatusCol+19);
        //if (io->state == true)
        if (io->value == true)
            vga.str("1");
        else
            vga.str("0");


        nRow++;

    }

    //Then, show Internal Relay status
    nRow++;
    SetRowCol(nRow++, nStatusCol);
    vga.str("\x1B[38;2;255;255;255m"); //foreground
    vga.str("\x1B[48;2;0;0;128m"); // background
    vga.str("Int. Relay STATUS:");
    SetRowCol(nRow++, nStatusCol);
    vga.str("NAME         VAL");
    vga.str("\x1B[38;2;255;255;255m"); //foreground
    vga.str("\x1B[48;2;0;0;0m"); // background

  
    int nintern = 0;
    while (nintern < MaxInternal)
    {//Update all IO states
        Internal* intern = InternalFromFile.p[nintern++];
        if (intern == 0)
            break;
        SetRowCol(nRow, nStatusCol);
        intern->row = nRow; //Save row and col so can toggle in simulation mode
        intern->col = nStatusCol;
        sprintf(mess, "%c %s", intern->type, intern->name);
        vga.str(mess);
        SetRowCol(nRow, nStatusCol + 14);
        if (intern->value == true)
            vga.str("1");
        else
            vga.str("0");


        nRow++;

    }



    //Show cursor as magenta block
    int x = MouseStatus.x / 8;
    int y = MouseStatus.y / 16;
    SetBackColor24(CursorColor);
    SetRowCol(y, x);
    vga.str(" ");
    int leftdown = MouseStatus.buttons & 1;
    if (leftdown)
    {
        mousedownx = x;
        mousedowny = y;
        mouseleftdown = true;
    }
    else
    {
        if ((mouseleftdown) && (mousedownx == x) && (mousedowny == y))
        {
            HandleMouseUp(pProgram, x, y);
        }
        mouseleftdown = false;
    }


    //printf("mouse at %d,%d, buttons=%x\n", x, y, MouseStatus.buttons);

    //Update visible screen
    vga.Update();
}



