#include "ladders.h"
#include "Platform.h"

ProgramObject ProgFromFile;

InternalList InternalFromFile;
IOList IoFromFile;



////////////////////////////////////////////////////////////////////////
//Count # of in and io found/created

int io = 0; //IO#
int nintern = 0; //Internal#

Internal* GetAssociatedInternal(char* name)
{//Scan internal list and find or create associated internal register
    for (int i = 0; i < nintern; i++)
    {
        Internal* intern = InternalFromFile.p[i];
        if (strcmp(name, intern->name) == 0)
        {
            printf("Found matching internal %s.\n", intern->name);
            return intern;
        }
            
    }
    //Still here? Need to create a new one
    
    Internal* intern = (Internal*)malloc(sizeof(Internal));
    InternalFromFile.p[nintern++] = intern;
    intern->name = name;
    intern->type = 'R';
    intern->value = 0;
    //printf("Created new Internal %s with value %d.\n", name, intern->value);
    return intern;
}

IO* GetAssociatedIO(char* name)
{//Scan internal list and find associated IO

    for (int i = 0; i < io; i++)
    {
        IO* pin = IoFromFile.p[i];
        if (strcmp(name, pin->name) == 0)
        {
            printf("Found matching pin %s.\n", pin->name);
            return pin;
        }
            
    }
    printf("Error, associated pin %s not found.\n", name);

}

void* GetAssociatedI(char source, char* name)
{//Find or create an associated IO pin or Internal object

    switch (source)
    {
    case 'R': //Internal
        //printf("returning internal\n");
        return (void*) GetAssociatedInternal(name);        
        break;
    case 'X': //IO input
    case 'Y': //IO output
        return (void*) GetAssociatedIO(name);
        break;
    default:
        break;
    }

}

ProgramObject* ParseLdFile(char* p)
{//Parse ld file stored in HUB RAM, pointed to by p
    char* line;
    char* token;
    char buf[256];
    bool bProgram = false;
    bool bIOList = false;
    
    
    int in = 0; //instruction #

    

    for (line = strtok(p, "\r\n"); line != NULL; line = strtok(line + strlen(line) + 1, "\r\n"))
    {
        strncpy(buf, line, sizeof(buf));
        //printf("Line: %s\n", buf);

        if ((!bIOList) && (!bProgram))
        {//Look for things like cycle time if not in IO List and not in Program List
            if (memcmp(buf, "CYCLE=", 6) == 0)
            {//Found cycle time directive
                token = strtok(buf, "=");
                token = strtok(token + strlen(token) + 1, " ");
                cycle_us = atoi(token);
                //printf("Cycle time=%d us\n", cycle_us); 
            }

        }

        if (strcmp(line, "IO LIST") == 0)
        {
            bIOList = true;
            continue;
        }

        if (bIOList)
        {
            token = strtok(buf, " ");
            if (strcmp(token, "END") == 0)
            {
                bIOList = false;
                continue;
            }

           // printf("IO = %s\n", token);
            //Create new IO
            IO* pIO = (IO*)malloc(sizeof(IO));
            IoFromFile.p[io++] = pIO;
            pIO->type = token[0];
            char* name = (char*)malloc(strlen(token)+1);
            strcpy(name, &token[1]);
            pIO->name = name;

            token = strtok(token + strlen(token) + 1, " ");
            if (strcmp(token, "at") != 0)
                printf("IO line error with %s\n",token);

            token = strtok(token + strlen(token) + 1, " ");
            pIO->pin = atoi(token);

            //pIO->state = 0;//start out off
            pIO->value = 0;//start out off

            printf("New IO of type %c, name %s, on pin #%d\n", pIO->type, pIO->name, pIO->pin);


        }



        if (strcmp(line, "PROGRAM") == 0)
        {
            bProgram = true;
            continue;
        }

        if (bProgram)
        {
            token = strtok(buf, " ");
            //printf("token = %s\n", token);
            if (strcmp(token, "RUNG") == 0)
            {//Create new ring
                RungObject* pRung = (RungObject*)malloc(sizeof(RungObject));
                ProgFromFile.p[in++] = (InstructionObject*)pRung;
                pRung->type = RungType;
                token = strtok(token + strlen(token) + 1, " ");
                pRung->nRung = atoi(token);
                //printf("Found Rung #%d\n", pRung->nRung);
            }
            else if (strcmp(token, "CONTACTS") == 0)
            {//Create new Contact
                ContactObject* pOb = (ContactObject*)malloc(sizeof(ContactObject));
                ProgFromFile.p[in++] = (InstructionObject*)pOb;
                pOb->type = ContactType;
                token = strtok(token + strlen(token) + 1, " ");
                pOb->source = token[0];
                char* name = (char*)malloc(strlen(token)+1);
                strcpy(name, &token[1]);
                pOb->name = name;

                //Find or create associated IO/Internal
                pOb->pI = GetAssociatedI(pOb->source, pOb->name);

                token = strtok(token + strlen(token) + 1, " ");
                pOb->bNegated = atoi(token);
                token = strtok(token + strlen(token) + 1, " ");
                pOb->bInitialValue = atoi(token);
                if (PlcMode == SimulationMode)
                    pOb->bValue = pOb->bInitialValue;
                else
                    pOb->bValue = 0;
                if (((IO*)pOb->pI)->type == 'X')
                {
                    //((IO*)pOb->pI)->newstate = pOb->bValue;
                    ((IO*)pOb->pI)->value = pOb->bValue;
                }
                //printf("Found Contact with source %c and name %s, Negated=%d, Init.Value=%d\n", pOb->source, pOb->name, pOb->bNegated, pOb->bValue);
            }
            else if (strcmp(token, "COIL") == 0)
            {//Create new Coil
                CoilObject* pOb = (CoilObject*)malloc(sizeof(CoilObject));
                ProgFromFile.p[in++] = (InstructionObject*)pOb;
                pOb->type = CoilType;
                token = strtok(token + strlen(token) + 1, " ");
                pOb->dest = token[0];
                char* name = (char*)malloc(strlen(token)+1);
                strcpy(name, &token[1]);
                pOb->name = name;

                pOb->bOn = false;
                pOb->bValue = 0;

                //Find or create associated IO/Internal
                pOb->pI = GetAssociatedI(pOb->dest, pOb->name);

                token = strtok(token + strlen(token) + 1, " ");
                pOb->bNegated = atoi(token);
                token = strtok(token + strlen(token) + 1, " ");
                pOb->bSetOnly = atoi(token);
                token = strtok(token + strlen(token) + 1, " ");
                pOb->bResetOnly = atoi(token);
                //printf("Found Coil with destination %c and name %s, Negated=%d, SetOnly=%d ResetOnly=%d\n", pOb->dest, pOb->name, pOb->bNegated, pOb->bSetOnly, pOb->bResetOnly);
            }
            else if (strcmp(token, "TON") == 0)
            {//Create new TON
                TonObject* pOb = (TonObject*)malloc(sizeof(TonObject));
                ProgFromFile.p[in++] = (InstructionObject*)pOb;
                pOb->type = TonType;
                token = strtok(token + strlen(token) + 1, " ");
                pOb->source = token[0];
                char* name = (char*)malloc(strlen(token) + 1);
                strcpy(name, &token[1]);
                pOb->name = name;

                //Find or create associated IO/Internal
                pOb->pI = GetAssociatedI(pOb->source, pOb->name);

                token = strtok(token + strlen(token) + 1, " ");
                pOb->onus = atoi(token);
                token = strtok(token + strlen(token) + 1, " ");
                pOb->adjust = atoi(token);
                
                //calculate cycle count
                pOb->oncycles = (pOb->onus / cycle_us) + pOb->adjust;
                pOb->count = 0;
                //printf("Found TON with source %c and name %s, On us=%d, adjust=%d, on cycles =%d \n", pOb->source, pOb->name, pOb->onus, pOb->adjust,pOb->oncycles);
               // for (;;) {}

            }
            else if (strcmp(token, "TOF") == 0)
            {//Create new TON
                ToffObject* pOb = (ToffObject*)malloc(sizeof(ToffObject));
                ProgFromFile.p[in++] = (InstructionObject*)pOb;
                pOb->type = ToffType;
                token = strtok(token + strlen(token) + 1, " ");
                pOb->source = token[0];
                char* name = (char*)malloc(strlen(token) + 1);
                strcpy(name, &token[1]);
                pOb->name = name;

                //Find or create associated IO/Internal
                pOb->pI = GetAssociatedI(pOb->source, pOb->name);

                token = strtok(token + strlen(token) + 1, " ");
                pOb->offus = atoi(token);
                token = strtok(token + strlen(token) + 1, " ");
                pOb->adjust = atoi(token);

                //calculate cycle count
                pOb->offcycles = (pOb->offus / cycle_us) + pOb->adjust;
                pOb->count = pOb->offcycles;//start out off 0;
                //printf("Found TOFF with source %c and name %s, On us=%d, adjust=%d, on cycles =%d \n", pOb->source, pOb->name, pOb->onus, pOb->adjust,pOb->oncycles);
               // for (;;) {}

            }
            else if (strcmp(token, "END") == 0)
            {//Create new END
                EndObject* pOb = (EndObject*)malloc(sizeof(EndObject));
                ProgFromFile.p[in++] = (InstructionObject*)pOb;
                pOb->type = EndType;
                //printf("Found End.\n");
            }
            else if (strcmp(token, "PARALLEL") == 0)
            {//Create new Parallel
                ParallelObject* pOb = (ParallelObject*)malloc(sizeof(ParallelObject));
                ProgFromFile.p[in++] = (InstructionObject*)pOb;
                pOb->type = ParallelType;
                //printf("Found Parallel.\n");
            }
            else if (strcmp(token, "SERIES") == 0)
            {//Create new Series
                SeriesObject* pOb = (SeriesObject*)malloc(sizeof(SeriesObject));
                ProgFromFile.p[in++] = (InstructionObject*)pOb;
                pOb->type = SeriesType;
                //printf("Found Series.\n");
            }
            else if (strcmp(token, "COMMENT") == 0)
            {//Create new Comment
                CommentObject* pOb = (CommentObject*)malloc(sizeof(CommentObject));
                ProgFromFile.p[in++] = (InstructionObject*)pOb;
                pOb->type = CommentType;
                // token = strtok(token + strlen(token) + 1, " ");
                char* com = token + 8;//first char in comment
                char* crlf = NULL;
                crlf = strstr(com, "\\r\\n");  
                if (crlf > 0)
                {//Second line in comment?
                    *crlf = 0; //end first string there
                    crlf += 4; //just past \r\n
                    char* comment2 = (char*)malloc(strlen(crlf) + 1);
                    strcpy(comment2, crlf);
                    pOb->line2 = comment2;
                }
                else
                    pOb->line2 = NULL;

                char* comment1 = (char*)malloc(strlen(com)+1);
                strcpy(comment1, com);
                pOb->line1 = comment1;
               // printf("Found Comment.\n");
            }
            else if (strcmp(token, "PLACEHOLDER") == 0)
            {//Create new Placeholder
                PlaceholderObject* pOb = (PlaceholderObject*)malloc(sizeof(PlaceholderObject));
                ProgFromFile.p[in++] = (InstructionObject*)pOb;
                pOb->type = PlaceholderType;
            }
            else
            {
                printf("Unknown instruction %s...\n", token);
            }

            /*
            for (token = strtok(buf, " "); token != NULL; token = strtok(token + strlen(token) + 1, " "))
            {
                printf("\tToken: %s\n", token);
            }
            */
        }
    }
    ProgFromFile.p[in] = NULL;  //NULL terminate list
    IoFromFile.p[io] = NULL;  //NULL terminate list

    return &ProgFromFile;

}


