#include "Platform.h"



#include "ladders.h"

//#include "ladderTest1.h"


//Spin2 VGA Driver
struct __using("ansi.spin2") vga; //RJA: Visual Studio will show an error here, but it's OK

//Using Spin2 to include .ld text file from ldmicro
struct __using("ldfile.spin2") ldfile; //RJA: Visual Studio will show an error here, but it's OK

//GarryJ's one cog USB Keyboard/Mouse driver
struct __using("1CogKbM_rja3c.spin2") kbm; //RJA:  Visual Studio will show an error here, but it's OK

//Some things are easier to do in Spin2
//struct __using("Helper.spin2") helper; //RJA:  Visual Studio will show an error here, but it's OK

typedef struct
{
    int x;
    int y;
    int MaxX;
    int MaxY;
    int buttons;
} MouseInfo;

MouseInfo MouseStatus = { 0,0,0,0 };
MouseInfo OldMouseStatus = { 0,0,0,0 };

int mousedownx;
int mousedowny;
bool mouseleftdown;

void StartMouse()
{//Start up the USB mouse
    printf("Starting Mouse...\n");
    MouseStatus.MaxX = 1024 - 1;
    MouseStatus.MaxY = 768 - 1;
    MouseStatus.x = 100;
    MouseStatus.y = 100;
    kbm.Start(USB_BASE_PIN, USB_ENABLE_PIN, USB_DM_PIN, USB_ERR_LED_PIN, &MouseStatus); 
    mouseleftdown = false;
}

ProgramObject* pPLC;

int nTerminalCogId;
int nRunCogId;

//Output screen to serial terminal?
bool bRunTerminal;



void setup()
{
    _setbaud(_BAUD); //in case we are using a faster baud, _BAUD defined in platform.h

    //Pick one:  EditMode, SimulationMode, RunMode
    PlcMode = SimulationMode;
    PlcMode = RunMode;

    //Output screen to serial terminal?
    bRunTerminal = true;
    //bRunTerminal = false; 

    
    vga.start(VGA_BASEPIN);
    StartMouse();

    cycle_us = 0;
    char* p = ldfile.getldfile();
    pPLC = ParseLdFile(p);
    if (cycle_us == 0)
    {//Must find cycle time to continue...
        printf("Error:  Cycle time not found! Stopping...\n");
        for (;;) {}
    }
    else
    {//Calculate #clocks in cycle
        //cycle_clks = ((cycle_us/1000) * (_clkfreq / 1000));
        cycle_clks = _muldiv64(cycle_us, _clkfreq, 1'000'000);
        
        printf("Cycle time is %d us (%d clocks with clockfreq=%d)\n",cycle_us, cycle_clks,_clkfreq);
        //for (;;) {}
    }

    


    //Start cog to stream screen over serial to ansi terminal
    static long stack[256];
    if (bRunTerminal)
        nTerminalCogId = __builtin_cogstart(TransmitScreen(), &stack[0]);



    PrepareBlankLadder(); //Create blank screen with just menu and ladder to speed screen drawing at expense of memory

    SetInitialValue(&ProgFromFile);

    
    //Start dedicated cog for run or simulate mode
    static long runstack[2000];
    if (PlcMode == RunMode)
        nRunCogId = __builtin_cogstart(runloop(), &runstack[0]);
    else if (PlcMode == SimulationMode)
        nRunCogId = __builtin_cogstart(simloop(), &runstack[0]);

}


int tstart;
int tend;
int telapsed;


void loop()
{  
    
    tstart = _getus();
    UpdateScreen(&ProgFromFile);// pPLC);  //Defined in LadderTest1.h
    tend = _getus();
    telapsed = (tend - tstart);    
    //_waitms(100);
    //tf("us Elapsed Updating screen= %d\n", telapsed);
    //for (;;) {}
}

void simloop()
{//Simulate PLC        
    _pinh(8);
    int clk_start;
    int clk_end = _getcnt();
    while (1)
    {
        clk_start = clk_end;
        clk_end = clk_start + cycle_clks;
        tstart = _getus();
        SimulateProgram(&ProgFromFile);
        tend = _getus();
        telapsed = (tend - tstart);
        //printf("us Elapsed Simulating program= %d\n", telapsed);
        //for (;;) {}
        //_pinnot(8); //testing cycle
        _waitcnt(clk_end);

    }

}

void runloop()
{//Run PLC        
    _pinh(8);
    int clk_start;
    int clk_end = _getcnt();
    while (1)
    {
        clk_start = clk_end;
        clk_end = clk_start + cycle_clks;
        tstart = _getus();
        RunProgram(&ProgFromFile);
        tend = _getus();
        telapsed = (tend - tstart);
        //printf("us Elapsed Running program= %d\n", telapsed);
        _pinnot(8); //testing cycle
        _waitcnt(clk_end);
        //for (;;) {}
    }

}



void main()
{//RJA added this main function to act like Arduino
    setup();
    printf("Setup complete.\n");
    

    for (;;)
    { 
        loop();
        nLoop++;
        

    }
}




