#ifndef _LADDERS_H_
#define _LADDERS_H_

#define MaxInstructions 1000

//#define MaxRungs 100
//#define MaxRungSteps 5
//#define MaxStepInstructions 10
#define MaxCommentLength 100
//#define MaxContacts 200
//#define MaxContactNameLength 200

int nLoop;
int PlcMode;
int cycle_us;
int cycle_clks;

enum PlcModes {
    EditMode = 0, SimulationMode = 1, RunMode
};

//vars shared by simulate and run modes
bool bSeriesOn;
bool bParallelOn;
bool bRungOn;


/////////////////////////////////////////////////////////////////////////////////////////
#define MaxIO (64)
#define MaxInternal (200)

typedef struct IO
{
    char type; // X or Y
    char* name; //name without prefix
    int value;  //for input: value read from pin if X
    int pin;  //IO pin#
    char row;
    char col;
}IO;

typedef struct IOList
{//Program rungs and other things
    //RungList* pRungList;   //pointer to NULL terminated list of Rungs
    IO* p[MaxIO+1];
} IOList;

typedef struct Internal
{
    char type;  // R
    char* name; //name without prefix
    int value; //value in memory
    char row;
    char col;
}Internal;

typedef struct InternalList
{//Program rungs and other things
    //RungList* pRungList;   //pointer to NULL terminated list of Rungs
    Internal* p[MaxInternal+1];
} InternalList;




/////////////////////////////////////////////////////////////////////////////////////////
typedef struct InstructionObject
{//Common beginning of all instruction objects
    int type;
} InstructionObject;

enum InstructionTypes {
    CommentType = 0, ContactType = 1, ParallelType, EndType, CoilType, RungType, SeriesType, PlaceholderType,
    TonType, ToffType
};

typedef struct RungObject
{//Start of Rung
    int type;
    int nRung; //Rung #
} RungObject;

typedef struct EndObject
{//End of something
    int type;
} EndObject;

typedef struct ParallelObject
{//Things in Parallel
    int type;
} ParallelObject;

typedef struct SeriesObject
{//Things in Series
    int type;
} SeriesObject;

typedef struct CommentObject
{//Comment instruction object
    int type;
    char* line1;
    char* line2;
} CommentObject;

typedef struct PlaceholderObject
{//Comment instruction object
    int type;
} PlaceholderObject;

typedef struct ContactObject
{//Contact instruction object
    int type;
    char source;
    char* name;// [MaxContactNameLength + 1] ;
    int bNegated;
    int bValue;
    int bInitialValue;
    bool bOn;
    bool bSeriesOn;
    void* pI; //pointer to related IO or Internal object
    char row;
    char col;
} ContactObject;

typedef struct TonObject
{//Ton instruction object
    int type;
    char source;
    char* name;// [MaxContactNameLength + 1] ;
    int onus;
    int adjust;
    int oncycles;
    int count;
    bool bOn;
    bool bSeriesOn;
    void* pI; //pointer to related IO or Internal object
    char row;
    char col;
} TonObject;

typedef struct ToffObject
{//Ton instruction object
    int type;
    char source;
    char* name;// [MaxContactNameLength + 1] ;
    int offus;
    int adjust;
    int offcycles;
    int count;
    bool bOn;
    bool bSeriesOn;
    void* pI; //pointer to related IO or Internal object
    char row;
    char col;
} ToffObject;

typedef struct CoilObject
{//Coil instruction object
    int type;
    char dest;
    char* name;// [MaxContactNameLength + 1] ;  //negated, &setOnly, &resetOnly, &ttrigger
    int bNegated;
    int bSetOnly;
    int bResetOnly;
    int bTrigger;
    int bValue;
    bool bOn;
    void* pI; //pointer to related IO or Internal object
    char row;
    char col;
} CoilObject;


typedef struct ProgramObject
{//Program rungs and other things
    //RungList* pRungList;   //pointer to NULL terminated list of Rungs
    InstructionObject* p[MaxInstructions+1];
} ProgramObject;



//Parser moved to seperate file for readability...
#include "Parser.c"

//LadderDrawing code also moved to seperate file
#include "DrawLadder.c"

//Simulation code is here for now...  Maybe needs to run in own cog?
#include "SimulateLadder.c"

//Run code is here
#include "RunLadder.c"

///////////////////////////////////////////////////////////////////////

#endif //_LADDERS_H_


